==> fispro_3.4.orig.tar.gz: fispro-3.4/cpp/fisopt/sampleB.cpp <==
//***********************************************************************
//
//
//
//
// Author(s) : Serge GUILLAUME, Brigitte CHARNOMORDIC 
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//**********************************************************************
/*!
 * \file sampleB.cpp
 * \brief sample of use of algo class whith a double Array ( C ).

 Optimisation of a double Array, using solis wetts algorythm.
 The evaluation function is the distance to NULL values of each row.

*/
#include <iostream.h>
#include <stdlib.h>
#include "algo.h"

// First to use an algorythm you need to write three functions :
// 1  code your object into an double array using a boolean key
// 2  code an double array into an object using a boolean key
// 3  the evaluation function of your object

// 1 ) This function update an Array whith the Object and the bollean key
// for an easier use of boolean key stored as an integer we will define
#define BOOL(x) (!(!(x)))
#define BitTst(arg,posn) BOOL((arg) & (1L << (posn)))
#define BitSet(arg,posn) ((arg) | (1L << (posn)))
#define BitClr(arg,posn) ((arg) & ~(1L << (posn)))

// to use your double array you need a size
#define MYSIZE 3
void   Print(double *,int );
int    sizeKey( int );
double Norme(double*);

void ObjectToDouble ( void * Item , int  Key , double *Array )
         {
         double *MyTab=( double * )Item;
         int j=0;
         for ( int i=0 ; i< MYSIZE ; i++ )
             { if ( BitTst( Key , i ) == 1 )
                { Array[i]=MyTab[j];
                  j++;
                  };
             };
#ifdef DEBUG3
         printf("\nObjectToDouble\nDouble genere\n");
         Print(Array,sizeKey(Key));
         printf("\nA partir de tableau courant et de Key %i\n",Key);
         Print(MyTab,MYSIZE);
#endif
         return ;
         };
// 2 ) This function update an Array whith the Object and the bollean key
void DoubleToObject ( double * Array , int size , int Key ,void * P )
         {
#ifdef DEBUG3
         printf("\nDoubleToObject\nDouble genere\n");
#endif
         double *MyTab=( double * )P;
         for ( int i=0 , j=0 ; i < MYSIZE ; i++ )
               {
               if ( BitTst( Key , i ) == 1 ) MyTab[i]=Array[j];
               j++;
               };
#ifdef DEBUG3
         Print(MyTab,MYSIZE);
         printf("\na partir de la cles %i et du double\n",Key);
         Print(Array,sizeKey(Key));
#endif
         return ;
        };

// 3 ) The evaluation function
// Second The evaluation function
// You can choose the name you want an pointer to tour function
// will be passed at the algorythm
// the function returns a double and have following params
double SampleTabEval(
// a pointer to an Object U try to optimize
                    void *  Item  ,
// which part of object U optimize
                    int  *  myKeyAdress ,
// array of double ( values )
                    double *Array ,
// Array size
                    int sizeArray ,
// A set of void * pointers to pass args to feval
                    conteneur * fEvalArgs )
       {
       int myKey=*myKeyAdress;
       char * passedChar;
       double result=-1;
// 3 operations :
//   cast void * to your type
//   rebuild object whith the key
//   get an evaluation of your object
      double *MyD=(double *) Item;
      DoubleToObject ( Array , sizeArray , myKey , MyD );
//   reach an fEvalArgs
     passedChar=(char*)fEvalArgs->get(1);
#ifdef DEBUG
     printf("\nParametre du main %s\n",passedChar);
#endif
//   get evaluation of updated object
        result=Norme(MyD);
        return result;
       };

int main(int argc, char *argv[])
{
  double *  MonTab   = new double[MYSIZE];
  double *  T        = new double[MYSIZE];
  double    gain;
  int       trouve , indice ;
// create a filename to pass to feval
  char      fEvalFileName[20];
  strcpy(fEvalFileName,"Nom.data\0");
// create an algo
  algo        *  A  = new algo( SOLISWETTS , MYSIZE , 1); // 1 param for feval
//  algo        *  A  = new algo( FORWARD , MYSIZE , 1 );
// change algo params
  A->P->maxIte=10;
  A->P->floor=1;
// change feval params
  indice=A->fEvalArgs->add(fEvalFileName);
// pour toutes les cles
for ( int cptK = 0 ; cptK < 8  ; cptK++ )
      {
// set key to the algo using decimal form
       A->setKey( cptK );
// as algo works on the item U pass this restore initial value of item
// after each step reinit the array
for ( int cpt2 = 0 ; cpt2 < MYSIZE  ; cpt2++ ) MonTab[cpt2]=cpt2;
// cast your object to an double
      ObjectToDouble ( MonTab , cptK , T );
      trouve=A->launch( MonTab , T , &gain, SampleTabEval );
#ifndef SPEEDY
    if ( trouve != 0 )
      { printf(" resultat trouve au bout de  %d iteration",trouve); }
      else
      { printf(" arret de l'algorithme apres %d iteration",A->P->maxIte); };
       Print(MonTab,MYSIZE);A->printKey();
       printf("\ncles %i Performance du vecteur transforme : %f \n-------------\n",cptK, gain );
#endif
    };

  delete A;
  delete [] MonTab;
  delete [] T;
  exit(0);
}

void Print(double *A,int size){
           printf("\n ");
           for ( int i=0; i< size ; i++ ) printf(" %f ",A[i]);
           return;
           }
int  sizeKey( int K ){
           int result=0;
           for ( int i=0; i< MYSIZE ; i++ )
           if ( BitTst(K,i)==1 ) result++;
           return result;
           }
double Norme(double *MyD){
           double result=0;
           for ( int i=0; i< MYSIZE ; i++ ) result += ( MyD[i]*MyD[i] );
           return sqrt( result );
           }

==> fispro_3.6.orig.tar.gz: fispro-3.6/cpp/fisopt/sampleB.cpp <==
//***********************************************************************
//
//
//
//
//
// Author(s) : Serge GUILLAUME, Brigitte CHARNOMORDIC 
// FISPRO Version : 3.2 - Copyright INRA - Cemagref - 2002 - IDDN.FR.001.030024.000.R.P.2005.003.31235
// Licence http://www.cecill.info/licences/Licence_CeCILL_V2-en.html/
// Last modification date:  July 31, 2009 - Contact : fispro@supagro.inra.fr
//**********************************************************************
/*!
 * \file sampleB.cpp
 * \brief sample of use of algo class whith a double Array ( C ).

 Optimisation of a double Array, using solis wetts algorythm.
 The evaluation function is the distance to NULL values of each row.

*/
#include <iostream>
#include <stdlib.h>
#include "algo.h"

// First to use an algorythm you need to write three functions :
// 1  code your object into an double array using a boolean key
// 2  code an double array into an object using a boolean key
// 3  the evaluation function of your object

// 1 ) This function update an Array whith the Object and the bollean key
// for an easier use of boolean key stored as an integer we will define
#define BOOL(x) (!(!(x)))
#define BitTst(arg,posn) BOOL((arg) & (1L << (posn)))
#define BitSet(arg,posn) ((arg) | (1L << (posn)))
#define BitClr(arg,posn) ((arg) & ~(1L << (posn)))

// to use your double array you need a size
#define MYSIZE 3
void   Print(double *,int );
int    sizeKey( int );
double Norme(double*);

//void ObjectToDouble ( void * Item , int  Key , double *Array )
void ObjectToDouble ( void * Item , vkey*  Key , double *Array )         {
         double *MyTab=( double * )Item;
         int j=0;
         for ( int i=0 ; i< MYSIZE ; i++ )
	   { //if ( BitTst( Key , i ) == 1 )
	     if ( Key->getValue(i) == 1 )
                { Array[i]=MyTab[j];
                  j++;
                  };
             };
         return ;
         };
// This function update an Object with the Array and the boolean key
//void DoubleToObject ( double * Array , int size , int Key ,void * P )
void DoubleToObject ( double * Array , int size , vkey* Key ,void * P )
         {
         double *MyTab=( double * )P;
         for ( int i=0 , j=0 ; i < MYSIZE ; i++ )
               {
		 //if ( BitTst( Key , i ) == 1 ) MyTab[i]=Array[j];
		 if ( Key->getValue(i) == 1 ) MyTab[i]=Array[j];
               j++;
               };
         return ;
        };

// 3 ) The evaluation function
// Second The evaluation function
// You can choose the name you want an pointer to tour function
// will be passed at the algorythm
// the function returns a double and have following params
double SampleTabEval(
// a pointer to an Object U try to optimize
                    void *  Item  ,
// which part of object U optimize
                    //int  *  myKeyAdress ,
		    vkey * mykey,
// array of double ( values )
                    double *Array ,
// Array size
                    int sizeArray ,
// A set of void * pointers to pass args to feval
                    conteneur * fEvalArgs )
       {
	 //int myKey=*myKeyAdress;

       char * passedChar;
       double result=-1;
// 3 operations :
//   cast void * to your type
//   rebuild object whith the key
//   get an evaluation of your object
      double *MyD=(double *) Item;
      DoubleToObject ( Array , sizeArray , mykey , MyD );
//   reach an fEvalArgs
     passedChar=(char*)fEvalArgs->get(1);

     //     printf("\nParametre du main %s\n",passedChar);

//   get evaluation of updated object
        result=Norme(MyD);
        return result;
       };

int main(int argc, char *argv[])
{
  double *  MonTab   = new double[MYSIZE];
  double *  T        = new double[MYSIZE];
  double    perf=-1.;
  int       trouve;
// create a filename to pass to feval
//  char      fEvalFileName[20];
//  strcpy(fEvalFileName,"Nom.data\0");
// create an algo
  algo        *  A  = new algo( SOLISWETTS , MYSIZE , 1); // 1 param for feval
//  algo        *  A  = new algo( FORWARD , MYSIZE , 1 );
  vkey key;

  A->P->maxIte=100;
 
  int cptK = 6;

  // for ( cptK = 0 ; cptK < 8  ; cptK++ )
  //        {
	key.setValue(cptK);
	A->setKey(& key);
	// A->setKey( cptK );
       // key=A->getKey();
       cout<<"\ncptK="<<cptK<<" taillecle="<<A->getKeySize()<<endl;
       key.Print();
       
// as algo works on the item U pass this restore initial value of item
// after each step reinit the array
for ( int cpt2 = 0 ; cpt2 < MYSIZE  ; cpt2++ ) MonTab[cpt2]=10.0 + 4.23*cpt2;
// cast your object to a double
//ObjectToDouble ( MonTab , cptK , T );
 ObjectToDouble ( MonTab , &key , T );
 cout <<"avant optim tableau MonTab, T"<<endl;
 for (int i=0 ; i < MYSIZE ; i++ )
   {
     cout<<MonTab[i]<<","<<T[i]<<"\t";
   }
 cout <<endl;
 
 //MonTab is modified, T is not
 trouve=A->launch( MonTab , T,&perf, SampleTabEval );
 cout <<"apres optim tableau MonTab, T"<<endl;
 for (int i=0 ; i < MYSIZE ; i++ )
   {
     cout<<MonTab[i]<<","<<T[i]<<"\t";
   }
 cout <<endl;
 
#ifndef SPEEDY
    if ( trouve != 0 )
      { printf(" resultat trouve au bout de  %d iteration",trouve); }
      else
      { printf(" arret de l'algorithme apres %d iterations",A->P->maxIte); };
    //       Print(MonTab,MYSIZE);A->printKey();
    
    printf("\nInitial norm:  %f,new norm: %f \n", Norme(T),perf);
#endif
       //	} // end of for

  delete A;
  delete [] MonTab;
  delete [] T;
  exit(0);
}

void Print(double *A,int size){
           printf("\n ");
           for ( int i=0; i< size ; i++ ) printf(" %f ",A[i]);
           return;
           }
int  sizeKey( int K ){
           int result=0;
           for ( int i=0; i< MYSIZE ; i++ )
           if ( BitTst(K,i)==1 ) result++;
           return result;
           }
double Norme(double *MyD){
           double result=0;
           for ( int i=0; i< MYSIZE ; i++ ) result += ( MyD[i]*MyD[i] );
           return sqrt( result );
           }
